@@ -14,18 +14,17 @@
 #ifndef CAML_CONFIG_H
 #define CAML_CONFIG_H
 
-#define ARCH_SIXTYFOUR
+#undef ARCH_SIXTYFOUR
 #define SIZEOF_INT 4
-#define SIZEOF_LONG 8
-#define SIZEOF_PTR 8
+#define SIZEOF_LONG 4
+#define SIZEOF_PTR 4
 #define SIZEOF_SHORT 2
-#define ARCH_INT64_TYPE long
-#define ARCH_UINT64_TYPE unsigned long
-#define ARCH_INT64_PRINTF_FORMAT "l"
+#define SIZEOF_LONGLONG 8
 #define ARCH_BIG_ENDIAN
-#undef ARCH_ALIGN_DOUBLE
-#undef ARCH_ALIGN_INT64
+#define ARCH_ALIGN_DOUBLE
+#define ARCH_ALIGN_INT64
 #undef NONSTANDARD_DIV_MOD
+#define ASM_CFI_SUPPORTED
 #define OCAML_OS_TYPE "Unix"
 #define OCAML_STDLIB_DIR "/usr/local/lib/ocaml"
 #define POSIX_SIGNALS
@@ -72,36 +71,18 @@
 #define SUPPORT_DYNAMIC_LINKING
 #define HAS_MMAP
 #define HAS_PWRITE
+#define HAS_MKSTEMP
+#define HAS_NICE
 #define HAS_SIGWAIT
+#define HAS_LIBBFD
 
 #ifndef CAML_NAME_SPACE
 #include "compatibility.h"
 #endif
 
-/* Types for signed chars, 32-bit integers, 64-bit integers,
+/* Types for 32-bit integers, 64-bit integers,
    native integers (as wide as a pointer type) */
 
-typedef signed char schar;
-
-#if SIZEOF_PTR == SIZEOF_LONG
-/* Standard models: ILP32 or I32LP64 */
-typedef long intnat;
-typedef unsigned long uintnat;
-#define ARCH_INTNAT_PRINTF_FORMAT "l"
-#elif SIZEOF_PTR == SIZEOF_INT
-/* Hypothetical IP32L64 model */
-typedef int intnat;
-typedef unsigned int uintnat;
-#define ARCH_INTNAT_PRINTF_FORMAT ""
-#elif SIZEOF_PTR == 8 && defined(ARCH_INT64_TYPE)
-/* Win64 model: IL32LLP64 */
-typedef ARCH_INT64_TYPE intnat;
-typedef ARCH_UINT64_TYPE uintnat;
-#define ARCH_INTNAT_PRINTF_FORMAT ARCH_INT64_PRINTF_FORMAT
-#else
-#error "No integer type available to represent pointers"
-#endif
-
 #if SIZEOF_INT == 4
 typedef int int32;
 typedef unsigned int uint32;
@@ -118,15 +99,40 @@ typedef unsigned short uint32;
 #error "No 32-bit integer type available"
 #endif
 
-#if defined(ARCH_INT64_TYPE)
+#ifndef ARCH_INT64_TYPE
+#if SIZEOF_LONGLONG == 8
+#define ARCH_INT64_TYPE long long
+#define ARCH_UINT64_TYPE unsigned long long
+#define ARCH_INT64_PRINTF_FORMAT "ll"
+#elif SIZEOF_LONG == 8
+#define ARCH_INT64_TYPE long
+#define ARCH_UINT64_TYPE unsigned long
+#define ARCH_INT64_PRINTF_FORMAT "l"
+#else
+#error "No 64-bit integer type available"
+#endif
+#endif
+
 typedef ARCH_INT64_TYPE int64;
 typedef ARCH_UINT64_TYPE uint64;
+
+#if SIZEOF_PTR == SIZEOF_LONG
+/* Standard models: ILP32 or I32LP64 */
+typedef long intnat;
+typedef unsigned long uintnat;
+#define ARCH_INTNAT_PRINTF_FORMAT "l"
+#elif SIZEOF_PTR == SIZEOF_INT
+/* Hypothetical IP32L64 model */
+typedef int intnat;
+typedef unsigned int uintnat;
+#define ARCH_INTNAT_PRINTF_FORMAT ""
+#elif SIZEOF_PTR == 8
+/* Win64 model: IL32LLP64 */
+typedef int64 intnat;
+typedef uint64 uintnat;
+#define ARCH_INTNAT_PRINTF_FORMAT ARCH_INT64_PRINTF_FORMAT
 #else
-#  ifdef ARCH_BIG_ENDIAN
-typedef struct { uint32 h, l; } uint64, int64;
-#  else
-typedef struct { uint32 l, h; } uint64, int64;
-#  endif
+#error "No integer type available to represent pointers"
 #endif
 
 /* Endianness of floats */
@@ -192,16 +198,17 @@ typedef struct { uint32 l, h; } uint64, int64;
 
 /* Minimum size increment when growing the heap (words).
    Must be a multiple of [Page_size / sizeof (value)]. */
-#define Heap_chunk_min (2 * Page_size / sizeof (value))
+#define Heap_chunk_min (15 * Page_size)
 
-/* Default size increment when growing the heap. (words)
-   Must be a multiple of [Page_size / sizeof (value)].
-   (Approx 512 Kb for a 32-bit platform, 1 Mb for a 64-bit platform.) */
-#define Heap_chunk_def (31 * Page_size)
+/* Default size increment when growing the heap.
+   If this is <= 1000, it's a percentage of the current heap size.
+   If it is > 1000, it's a number of words. */
+#define Heap_chunk_def 15
 
 /* Default initial size of the major heap (words);
-   same constraints as for Heap_chunk_def. */
+   Must be a multiple of [Page_size / sizeof (value)]. */
 #define Init_heap_def (31 * Page_size)
+/* (about 512 kB for a 32-bit platform, 1 MB for a 64-bit platform.) */
 
 
 /* Default speed setting for the major GC.  The heap will grow until
