@@ -1,6 +1,6 @@
 (***********************************************************************)
 (*                                                                     *)
-(*                           Objective Caml                            *)
+(*                                OCaml                                *)
 (*                                                                     *)
 (*            Xavier Leroy, projet Cristal, INRIA Rocquencourt         *)
 (*                                                                     *)
@@ -11,8 +11,6 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: obj.mli,v 1.30 2008/01/29 13:11:15 doligez Exp $ *)
-
 (** Operations on internal representations of values.
 
    Not for the casual user.
@@ -28,11 +26,15 @@ external is_int : t -> bool = "%obj_is_int"
 external tag : t -> int = "caml_obj_tag"
 external set_tag : t -> int -> unit = "caml_obj_set_tag"
 external size : t -> int = "%obj_size"
-external truncate : t -> int -> unit = "caml_obj_truncate"
 external field : t -> int -> t = "%obj_field"
 external set_field : t -> int -> t -> unit = "%obj_set_field"
+val double_field : t -> int -> float  (* @since 3.11.2 *)
+val set_double_field : t -> int -> float -> unit  (* @since 3.11.2 *)
 external new_block : int -> int -> t = "caml_obj_block"
 external dup : t -> t = "caml_obj_dup"
+external truncate : t -> int -> unit = "caml_obj_truncate"
+external add_offset : t -> Int32.t -> t = "caml_obj_add_offset"
+         (* @since 3.12.0 *)
 
 val lazy_tag : int
 val closure_tag : int
@@ -41,18 +43,25 @@ val infix_tag : int
 val forward_tag : int
 val no_scan_tag : int
 val abstract_tag : int
-val string_tag : int
+val string_tag : int   (* both [string] and [bytes] *)
 val double_tag : int
 val double_array_tag : int
 val custom_tag : int
-val final_tag : int  (* DEPRECATED *)
+val final_tag : int
+  [@@ocaml.deprecated "Replaced by custom_tag."]
 
 val int_tag : int
 val out_of_heap_tag : int
-val unaligned_tag : int   (* should never happen *)
+val unaligned_tag : int   (* should never happen @since 3.11.0 *)
+
+val extension_name : 'a -> string
+val extension_id : 'a -> int
+val extension_slot : 'a -> t
 
 (** The following two functions are deprecated.  Use module {!Marshal}
     instead. *)
 
-val marshal : t -> string
-val unmarshal : string -> int -> t * int
+val marshal : t -> bytes
+  [@@ocaml.deprecated "Use Marshal.to_bytes instead."]
+val unmarshal : bytes -> int -> t * int
+  [@@ocaml.deprecated "Use Marshal.from_bytes and Marshal.total_size instead."]
